import os, sys
import numpy as np
import ase.io
import ase.build
import dataset
import copy
import pickle
from scipy.optimize import curve_fit

database_file="job_database_H2onCu_flat.db"

def get_geom(geom, vacuum, layers):
    geom_text=get_orig_geom(geom)
    if (vacuum == 'standard' or vacuum == 13) and (layers == 'standard' or layers == 4):
        return geom_text
    else:
        #adjust number of layers
        f = open('tmp', 'w')
        f.write(geom_text)
        f.close()
        atoms = ase.io.read('tmp', format='vasp')
        os.remove('tmp')
        if layers in [3]:
            #delete layers that are not needed
            height = set(atoms.positions[:,2])
            height = list(height)
            del_height = height[0]
            del atoms[[atom.index for atom in atoms if atom.position[2] in del_height]]
            #adjust positions
            for atom in atoms:
                atom.positions[2]=atom.positions[2]-height[1]+height[0]
            #adjust vacuum height
            atoms.cell[2,2]=atoms.cell[2,2]-height[1]+height[0]
        if not (vacuum == 'standard' or vacuum == 13):
            #shift all atoms down
            atoms.positions[:,2] = atoms.positions[:,2]-6+vacuum/2.
            #adjust vacuum height
            atoms.cell[2,2]=atoms.cell[2,2]-12+vacuum
            #if geom='asy': shift H atoms
            if geom == 'asy':
                atoms.positions[-1,2]=atoms.positions[-1,2]-6+vacuum/2.
                atoms.positions[-2,2]=atoms.positions[-2,2]-6+vacuum/2.
        filename = 'tmp'
        ase.io.write(filename, atoms, format = 'vasp')
        with open(filename, 'r') as f:
            geom_text = f.read()
        os.remove(filename)
        return geom_text


def get_orig_geom(geom):
    #geometry taken from analysis_04.ipynb as used for the analysis of the QMC results in the 2017 paper of H2 on Cu(111)
    positionsDict={}
    positionsDict['asy']=[]
    positionsDict['asy'].append({"atom":'Cu', "position":[   0.000000000,   0.000000000,   0.000000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   2.552655436,   0.000000000,   0.000000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[  -1.276327718,   2.210664573,   0.000000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   1.276327718,   2.210664573,   0.000000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   1.276327756,   0.736888213,  -2.063000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   3.828983193,   0.736888213,  -2.063000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   0.000000038,   2.947552786,  -2.063000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   2.552655474,   2.947552786,  -2.063000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   0.000000000,   1.473776426,  -4.143000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   2.552655436,   1.473776426,  -4.143000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[  -1.276327718,   3.684440999,  -4.143000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   1.276327718,   3.684440999,  -4.143000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   0.000000000,   0.000000000,  -6.227000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   2.552655436,   0.000000000,  -6.227000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[  -1.276327718,   2.210664573,  -6.227000000]})
    positionsDict['asy'].append({"atom":'Cu', "position":[   1.276327718,   2.210664573,  -6.227000000]})
    positionsDict['ts1']=copy.deepcopy(positionsDict['asy'])
    positionsDict['asy'].append({"atom":'H', "position":[    1.276327702,    .370720000,   6.499999984]})
    positionsDict['asy'].append({"atom":'H', "position":[    1.276327735,   -.370720000,   6.500000016]})
    positionsDict['ts1'].append({"atom":'H', "position":[       1.276327696,   0.516000000,   1.163999977]})
    positionsDict['ts1'].append({"atom":'H', "position":[       1.276327741,  -0.516000000,   1.164000023]})

    v1=[ 5.105310873,    0.000000000,    0.000000000]
    v2=[-2.552655436,    4.421329147,    0.000000000]
    v3=[ 0.000000000,    0.000000000,   19.227000000]

    #shift atom into center of simulation cell in z direction (according to asymt geometry)
    z=[]
    for atom in positionsDict['asy']:
        z.append(atom['position'][2])
    shiftz=v3[2]/2-(min(z)+max(z))/2

    for atom in positionsDict[geom]:
        atom['position'][2]=atom['position'][2]+shiftz

    #transfer to an ase geometry
    atoms=ase.Atoms('Cu16H2', positions = [atom['position'] for atom in positionsDict[geom]], cell = [v1,v2,v3])

    filename = 'tmp'
    ase.io.write(filename, atoms, format = 'vasp')
    with open(filename, 'r') as f:
        geom_text = f.read()
    os.remove(filename)
    return geom_text


def get_results(geom, layer_list, vacuum_list, k_list, ecut_list):
    #check that only one list has a length longer than 1
    nLists = 0
    for mylist in [layer_list, vacuum_list, k_list, ecut_list]:
        if len(mylist)>1:
            nLists +=1
    if nLists > 1:
        sys.exit("too many lists in function get_results")

    #define some variables to get rpa results
    xc = 'rpar_4step'
    smearing = 'fermi'
    smearing_val = 0.1 #eV
    nomega = 12
    finite_temperature = True
    pseudo = "POTCAR_Cu_H"

    #get the results
    results = []
    for layers in layer_list:
        for vacuum in vacuum_list:
            for ks in k_list:
                for ecut in ecut_list:
                    energy={}
                    for config in ['asy', geom]:
                        geom_text = get_geom(config,vacuum,layers)
                        #create job record
                        job_record={'program': 'vasp6.3.0',\
                                'scf': 'single point',\
                                'nkx': ks,\
                                'nky': ks,\
                                'nkz':1,\
                                'ecut':ecut,\
                                'xc':xc,\
                                'smearing':smearing,\
                                'smearing val':smearing_val,\
                                'nomega': nomega,\
                                'finite temperature': finite_temperature,\
                                'geom':geom_text,\
                                'pseudofiles': pseudo}        
                        #retreive job from db
                        database = dataset.connect('sqlite:///'+database_file)
                        jobs = list(database['data'].find(**job_record))
                        if len(jobs)==1:
                            if jobs[0]['status']=='done':
                                energy[config]=jobs[0]['energy']
                    try:
                        results.append((float(energy[geom])-float(energy['asy'])))
                    except:
                        results.append(float('nan'))
    return results

def get_extrap(xdata, ydata):
    xstart = 400
    def ecut2N(ecut):
        ecut = np.array(ecut)
        return (ecut/xstart)**(-5./2.)

    def func(x, a, b):
        x = np.array(x)
        return a*x + b

    #shift data
    shift = ydata[0]
    if not xdata[0]==xstart:
        sys.exit("missing a value!!!")
    for i in range(len(ydata)):
        ydata[i]=ydata[i]-shift
    #fit
    popt, pcov = curve_fit(func, ecut2N(xdata), ydata)
    return popt[1]
    
values = {}
values['base'] = []
values['ecutCorr1'] = []
values['ecutCorr2'] = []
values['kCorr1'] = []
values['kCorr2'] = []
values['final'] = []

geom_list = ['ts1']
for geom in geom_list:
    print("-----------{}---------------".format(geom))
    #ecut extrap1
    print("   ecut extrapolation 1: (l=standard, v=standard, ecut=400) --> (l=standard, v=standard, ecut=550) --> (l=standard, v=standard, ecut=inf) @k=5")
    layer_list = ['standard']
    vacuum_list = ['standard']
    k_list = [5]
    ecut_list = [400,450,500,550]
    results = get_results(geom, layer_list, vacuum_list, k_list, ecut_list)
    print()
    print("   ecuts used in extrap: {}".format(ecut_list))
    print("   barriers used in ecut extrap: {}".format(results))
    if any(np.isnan(results)):
        values['kCorr1'].append(np.nan)
        values['base'].append(np.nan)
        values['final'].append(np.nan)
        print("Not all results found. Cycling.")
        continue
    ecutCorr1 = get_extrap(ecut_list, results)
    values['ecutCorr1'].append(ecutCorr1)
    print("   ecutCorr1={}".format(ecutCorr1))

    #kpoint extrap1
    print()
    print("   k-point extrapolation 1: (l=standard, v=standard, k=5) --> (l=standard, v=standard, k=12) @ecut=300") 
    layer_list = ['standard']
    vacuum_list = ['standard']
    k_list = [5,12]
    ecut_list = [300]
    results = get_results(geom, layer_list, vacuum_list, k_list, ecut_list)
    print("   barriers used in k-point extrap1: {}".format(results))
    if any(np.isnan(results)):
        values['kCorr1'].append(np.nan)
        values['base'].append(np.nan)
        values['final'].append(np.nan)
        print("Not all results found. Cycling.")
        continue
    kCorr1 = results[1]-results[0]
    values['kCorr1'].append(kCorr1)
    print("   kCorr1={}".format(kCorr1))

    #base calc
    print()
    print("   base calculation: (l=standard, v=standard, k=5, ecut=400)")
    layer_list = ['standard']
    vacuum_list = ['standard']
    k_list = [5]
    ecut_list = [400]
    results = get_results(geom, layer_list, vacuum_list, k_list, ecut_list)
    if any(np.isnan(results)):
        values['base'].append(np.nan)
        values['final'].append(np.nan)
        print("Not all results found. Cycling.")
        continue
    base = results[0]
    values['base'].append(base)

    print("")
    print("   base value (l=10, vac=24, ecut=250, k=4): {}".format(base))
    print("")
    final = base+ecutCorr1+kCorr1
    values['final'].append(final)
    print("corrected value: {}".format(final))

print()
print("-----------------summary--------------------")
line = "\t\t"
for geom in geom_list:
    line=line+"  {}\t".format(geom)
print(line)
for desc in ['base', 'ecutCorr1',  'kCorr1',  'final']:
    line = "{:10s}\t".format(desc)
    for i,geom in enumerate(geom_list):
        line=line+"{:7.4f}\t".format(values[desc][i])
    if desc == 'final':
        print("_________________________________________________________")
    print(line)

barriers_H2onCu_RPA = {}
barriers_H2onCu_RPA['rpa']=values['final'][0]
f = open('H2onCu_RPA.pkl', 'wb')
pickle.dump(barriers_H2onCu_RPA, f)
f.close()


